comment = 'comment'
pointing_hms_dms = 'pointing_hms_dms'
pointing_degrees = 'pointing_degrees'
calibrator = 'calibrator'
rain = 'rain_amp_corr'
flagged_max = 'flagged_max_percent'
flagged_final = 'flagged_final_percent'
est_noise_jy = 'estimated_noise_jy'
target_uvfits = 'target_uvfits'
cal_uvfits = 'calib_uvfits'
files = 'files'
field = 'field'
target_pointing_deg = 'target_median_pointing'
group_name = 'group_name'
obs_name = 'obs_name'
time_ut = 'time_utc'
time_mjd = 'time_mjd'
time_st = 'time_sidereal'
duration = 'duration_hrs'
archive_cal_available = 'archive_cal_available'
archive_cal_days_apart = 'archive_cal_days_apart'
raw_obs_text = 'raw_obs_listing_text'
warnings = 'warning_flags'
warning_incomplete = 'incomplete_observation'
